function Tpoly = tesselatePoly(poly,order)
% function tesselatePoly(poly,order)
% 
% Tesselated each face of the input polyhedron, poly, to 
% generate a new Polyhedron.
%
% (C) Copyright, 2006 Bennett Landman, bennett@bme.jhu.edu
% Distributed: July 19, 2006

if(~exist('order'))
    order=2;
end
if(order<2)
    order=2;
    disp('Warning: Minimum tesselation order is 2 => 2 divisions per side.');
end
vert = poly.vert;
face = poly.face;
outFace = {};
for jF=1:length(face)
    if(length(face{jF})>3)
        FaceCenter = mean(vert(face{jF},:));
        [vert, FaceCenterIdx] = addVertex(vert,FaceCenter);
        oF={};
        oF{end+1} = [face{jF}(end) face{jF}(1) FaceCenterIdx];
        for jV=2:length(face{jF})
            oF{end+1} =  [face{jF}(jV-1) face{jF}(jV) FaceCenterIdx];
        end
        if(order>2)
            for jV=1:length(oF)
                [vert,outFace] = tesselateTri(vert,outFace, oF{jV}, order-1);
            end
        else
            outFace = [outFace oF];
        end
            
        
    else
        if(length(face{jF})==3)
            [vert,outFace] = tesselateTri(vert,outFace, face{jF}, order);
        else
            error('Unsupported face type. Too few sides?');
        end
    end
end
Tpoly.vert = vert;
Tpoly.face = outFace;

function [vert,outFace] = tesselateTri(vert,outFace, face, order)
if(order==2)
    Vab = mean(vert(face(1:2),:));
    [vert, iVab] = addVertex(vert,Vab);
    Vbc = mean(vert(face(2:3),:));
    [vert, iVbc] = addVertex(vert,Vbc);
    Vac = mean(vert(face([1 3]),:));
    [vert, iVac] = addVertex(vert,Vac);
    outFace{end+1} = [face(1) iVab iVac];
    outFace{end+1} = [face(2) iVab iVbc];
    outFace{end+1} = [face(3) iVac iVbc];
    outFace{end+1} = [iVab iVbc iVac];
else
    Vab_n = 1/order*vert(face(1),:)+(order-1)/order*vert(face(2),:);
    [vert, iVab_n] = addVertex(vert,Vab_n);
    Vac_n = 1/order*vert(face(1),:)+(order-1)/order*vert(face(3),:);
    [vert, iVac_n] = addVertex(vert,Vac_n);
    % Tesselate the sub problem
    [vert,outFace] = tesselateTri(vert,outFace, [face(1) iVab_n iVac_n], order-1);
    
    % Add the triangle at the top of the ladder
    dBC = (vert(face(3),:)-vert(face(2),:))/order;
    Vbc_n = vert(face(2),:)+dBC;
    [vert, iVbc_n] = addVertex(vert,Vbc_n);
    outFace{end+1} = [face(2) iVab_n iVbc_n];
    
    Vbc0_n = Vab_n;
    iVbc0_n = iVab_n;
    
    % Walk down the ladder adding two triangles at once
    for j=1:(order-1)
        Vbc_n2 = Vbc_n+dBC;
        [vert, iVbc_n2] = addVertex(vert,Vbc_n2);
        Vbc0_n2 = Vbc0_n+dBC;
        [vert, iVbc0_n2] = addVertex(vert,Vbc0_n2);
        outFace{end+1} = [iVbc_n iVbc_n2 iVbc0_n2];
        outFace{end+1} = [iVbc_n iVbc0_n iVbc0_n2];
        
        iVbc_n = iVbc_n2; Vbc_n=Vbc_n2;
        iVbc0_n = iVbc0_n2; Vbc0_n = Vbc0_n2;
    end

end

function [vert, iV] = addVertex(vert,V)
iV = find(sum(abs(vert-repmat(V,[size(vert,1) 1])),2)<1e-4);
if(isempty(iV))
    vert(end+1,:) = V;
    iV = length(vert);
end